@@ -24,6 +24,7 @@
 # - Tobias Wegner <twegner@cern.ch>, 2017
 # - Brian Bockelman <bbockelm@cse.unl.edu>, 2017-2018
 # - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
+# - Jaroslav Guenther <jaroslav.guenther@gmail.com>, 2019-2020
 #
 # PY3K COMPATIBLE
 
@@ -36,6 +37,13 @@ from __future__ import print_function
 import imp
 import random
 import sys
+import traceback
+import time
+
+from logging import getLogger, StreamHandler, ERROR
+from os import environ, fdopen, path, makedirs, geteuid
+from shutil import move
+from tempfile import mkstemp
 
 from rucio.common import exception
 from rucio.common.config import config_get
@@ -45,10 +53,6 @@ from rucio.common.exception import (CannotAuthenticate, ClientProtocolNotSupport
 from rucio.common.utils import build_url, get_tmp_dir, my_key_generator, parse_response, ssh_sign
 from rucio import version
 
-from logging import getLogger, StreamHandler, ERROR
-from os import environ, fdopen, path, makedirs, geteuid
-from shutil import move
-from tempfile import mkstemp
 try:
     # Python 2
     from urlparse import urlparse
@@ -60,7 +64,7 @@ except ImportError:
 from dogpile.cache import make_region
 from requests import Session
 from requests.status_codes import codes, _codes
-from requests.exceptions import ConnectionError
+from requests.exceptions import ConnectionError, RequestException
 from requests.packages.urllib3 import disable_warnings  # pylint: disable=import-error
 disable_warnings()
 
@@ -154,7 +158,7 @@ class BaseClient(object):
             LOG.debug('no auth_type passed. Trying to get it from the environment variable RUCIO_AUTH_TYPE and config file.')
             if 'RUCIO_AUTH_TYPE' in environ:
                 if environ['RUCIO_AUTH_TYPE'] not in ['userpass', 'x509', 'x509_proxy', 'gss', 'ssh', 'saml']:
-                    raise MissingClientParameter('Possible RUCIO_AUTH_TYPE values: userpass, x509, x509_proxy, gss, ssh, saml, vs. ' + environ['RUCIO_AUTH_TYPE'])
+                    raise MissingClientParameter('Possible RUCIO_AUTH_TYPE values: userpass, x509, x509_proxy, gss, ssh, saml, oidc, vs. ' + environ['RUCIO_AUTH_TYPE'])
                 self.auth_type = environ['RUCIO_AUTH_TYPE']
             else:
                 try:
@@ -169,6 +173,16 @@ class BaseClient(object):
                 if self.auth_type in ['userpass', 'saml']:
                     self.creds['username'] = config_get('client', 'username')
                     self.creds['password'] = config_get('client', 'password')
+                elif self.auth_type == 'oidc':
+                    self.creds['oidc_auto'] = config_get('client', 'oidc_auto')
+                    self.creds['oidc_scope'] = config_get('client', 'oidc_scope')
+                    self.creds['oidc_audience'] = config_get('client', 'oidc_audience')
+                    self.creds['oidc_polling'] = config_get('client', 'oidc_polling')
+                    self.creds['oidc_refresh_lifetime'] = config_get('client', 'oidc_refresh_lifetime')
+                    self.creds['oidc_issuer'] = config_get('client', 'oidc_issuer')
+                    if self.creds['oidc_auto']:
+                        self.creds['oidc_username'] = config_get('client', 'oidc_username')
+                        self.creds['oidc_password'] = config_get('client', 'oidc_password')
                 elif self.auth_type == 'x509':
                     self.creds['client_cert'] = path.abspath(path.expanduser(path.expandvars(config_get('client', 'client_cert'))))
                     self.creds['client_key'] = path.abspath(path.expanduser(path.expandvars(config_get('client', 'client_key'))))
@@ -361,6 +375,121 @@ class BaseClient(object):
         LOG.debug('got new token')
         return True
 
+    def __get_token_OIDC(self):
+        """
+        First authenticates the user via a Identity Provider server
+        (with user's username & password), by specifying oidc_scope,
+        user agrees to share the relevant information with Rucio.
+        If all proceeds well, an access token is requested from the Identity Provider.
+        Access Tokens are not stored in Rucio DB.
+        Refresh Tokens are granted only in case no valid access token exists in user's
+        local storage, oidc_scope includes 'offline_access'. In such case, refresh token
+        is stored in Rucio DB.
+
+        :returns: True if the token was successfully received. False otherwise.
+        """
+        oidc_scope = str(self.creds['oidc_scope'])
+        headers = {'X-Rucio-Account': self.account,
+                   'X-Rucio-Client-Authorize-Auto': str(self.creds['oidc_auto']),
+                   'X-Rucio-Client-Authorize-Polling': str(self.creds['oidc_polling']),
+                   'X-Rucio-Client-Authorize-Scope': str(self.creds['oidc_scope']),
+                   'X-Rucio-Client-Authorize-Refresh-Lifetime': str(self.creds['oidc_refresh_lifetime'])}
+        if self.creds['oidc_audience']:
+            headers['X-Rucio-Client-Authorize-Audience'] = str(self.creds['oidc_audience'])
+        if self.creds['oidc_issuer']:
+            headers['X-Rucio-Client-Authorize-Issuer'] = str(self.creds['oidc_issuer'])
+        if self.creds['oidc_auto']:
+            userpass = {'username': self.creds['oidc_username'], 'password': self.creds['oidc_password']}
+        for retry in range(self.AUTH_RETRIES + 1):
+            try:
+                start = time.time()
+                result = None
+                request_auth_url = build_url(self.auth_host, path='auth/oidc')
+                # requesting authorization URL specific to the user & Rucio OIDC Client
+                OIDC_auth_res = self.session.get(request_auth_url, headers=headers, verify=self.ca_cert)
+                # with the obtained authorization URL we will contact the Identity Provider to get to the login page
+                if 'X-Rucio-OIDC-Auth-URL' not in OIDC_auth_res.headers:
+                    print("Rucio Client did not succeed to get AuthN/Z URL from the Rucio Auth Server. \
+                           \nThis could be due to wrongly requested/configured scope, audience of issuer.")
+                    return False
+                auth_url = OIDC_auth_res.headers['X-Rucio-OIDC-Auth-URL']
+                if not self.creds['oidc_auto']:
+                    print("\nYou chose to authenticate using your internet browser.\n"  # NOQA: W503
+                          + "You can also use --auto option and trust Rucio Client \n"  # NOQA: W503
+                          + "with your IdP credentials to proceed for \n"  # NOQA: W503
+                          + "automatic IdP log-in from CLI.\n")  # NOQA: W503
+                    print("--------------------------------------------------")
+                    print("Please use your internet browser, go to:")
+                    print("\n    " + auth_url + "    \n")
+                    print("and authenticate with your Identity Provider.")
+
+                    headers['X-Rucio-Client-Fetch-Token'] = 'True'
+                    if self.creds['oidc_polling']:
+                        timeout = 180
+                        start = time.time()
+                        print("3 minutes from now the authentication attempt will time out.")
+                        while time.time() - start < timeout:
+                            result = self.session.get(auth_url, headers=headers, verify=self.ca_cert)
+                            if 'X-Rucio-Auth-Token' in result.headers and result.status_code == codes.ok:
+                                break
+                            time.sleep(2)
+                    else:
+                        print("Copy paste the code from the browser to the terminal and press enter:")
+                        count = 0
+                        while count < 3:
+                            fetchcode = raw_input()
+                            fetch_url = build_url(self.auth_host, path='auth/oidc_redirect', params=fetchcode)
+                            result = self.session.get(fetch_url, headers=headers, verify=self.ca_cert)
+                            if 'X-Rucio-Auth-Token' in result.headers and result.status_code == codes.ok:
+                                break
+                            else:
+                                print("The Rucio Auth Server did not respond as expected. Please, "
+                                      + "try again and make sure you typed the correct code.")  # NOQA: W503
+                                count += 1
+                else:
+                    auth_res = self.session.get(auth_url, verify=self.ca_cert)
+                    # getting the login URL and logging in the user
+                    login_url = auth_res.url
+                    start = time.time()
+                    result = self.session.post(login_url, data=userpass, verify=self.ca_cert, allow_redirects=True)
+                    # if the Rucio OIDC Client configuration does not match the one registered at the Identity Provider
+                    # the user will get an OAuth error
+                    if 'OAuth Error' in result.text:
+                        LOG.error('Identity Provider does not allow to proceed. Could be due to misconfigured redirection server name of the Rucio OIDC Client.')
+                        return False
+                    # In case Rucio Client is not authorized to request information about this user yet,
+                    # it will automatically authorize itself on behalf of the user.
+                    if result.url == auth_url:
+                        form_data = {}
+                        for scope_item in oidc_scope.split():
+                            form_data["scope_" + scope_item] = scope_item
+                        default_data = {"remember": "until-revoked",
+                                        "user_oauth_approval": True,
+                                        "authorize": "Authorize"}
+                        form_data.update(default_data)
+                        LOG.warning('Automatically authorising request of the following info on behalf of user: %s', str(form_data))
+                        # authorizing info request on behalf of the user until he/she revokes this authorization !
+                        result = self.session.post(result.url, data=form_data, verify=self.ca_cert, allow_redirects=True)
+                break
+            except RequestException:
+                LOG.warning('RequestException: %s', str(traceback.format_exc()))
+                self.ca_cert = False
+                if retry > self.request_retries:
+                    raise
+
+        if not result or 'result' not in locals():
+            LOG.error('cannot get auth_token')
+            return False
+
+        if result.status_code != codes.ok:  # pylint: disable-msg=E1101
+            exc_cls, exc_msg = self._get_exception(headers=result.headers,
+                                                   status_code=result.status_code,
+                                                   data=result.content)
+            raise exc_cls(exc_msg)
+
+        self.auth_token = result.headers['x-rucio-auth-token']
+        return True
+
     def __get_token_x509(self):
         """
         Sends a request to get an auth token from the server and stores it as a class attribute. Uses x509 authentication.
@@ -597,6 +726,10 @@ class BaseClient(object):
                 if not self.__get_token_x509():
                     raise CannotAuthenticate('x509 authentication failed for account=%s with identity=%s' % (self.account,
                                                                                                              self.creds))
+            elif self.auth_type == 'oidc':
+                if not self.__get_token_OIDC():
+                    raise CannotAuthenticate('OIDC authentication failed for account=%s' % self.account)
+
             elif self.auth_type == 'gss':
                 if not self.__get_token_gss():
                     raise CannotAuthenticate('kerberos authentication failed for account=%s with identity=%s' % (self.account,
@@ -676,6 +809,11 @@ class BaseClient(object):
         if self.auth_type == 'userpass':
             if self.creds['username'] is None or self.creds['password'] is None:
                 raise NoAuthInformation('No username or password passed')
+        elif self.auth_type == 'oidc':
+            if self.creds['oidc_auto'] and (self.creds['oidc_username'] is None or self.creds['oidc_password'] is None):
+                raise NoAuthInformation('For automatic OIDC log-in with your Identity Provider username and password are required.')
+            if not self.creds['oidc_scope']:
+                raise NoAuthInformation('For OIDC log-in you need to provide a scope parameter. The minimal expected by Rucio server is usually "openid profile"')
         elif self.auth_type == 'x509':
             if self.creds['client_cert'] is None:
                 raise NoAuthInformation('The path to the client certificate is required')
