@@ -1,20 +1,35 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
-# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Thomas Beermann, <thomas.beermann@cern.ch>, 2012
-# - Vincent Garonne, <vincent.garonne@cern.ch>, 2012-2013
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2013
-# - Ralph Vigne, <ralph.vigne@cern.ch>, 2013-2014
-# - Martin Barisits, <martin.barisits@cern.ch>, 2013
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2014
+# - Vincent Garonne <vgaronne@gmail.com>, 2012-2018
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2012
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2018
+# - Ralph Vigne <ralph.vigne@cern.ch>, 2013-2015
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2018
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2014
+# - Wen Guan <wguan.icedew@gmail.com>, 2014
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+#
+# PY3K COMPATIBLE
 
 from json import dumps, loads
 from requests.status_codes import codes
-from urllib import quote
+try:
+    from urllib import quote
+except ImportError:
+    from urllib.parse import quote
 
 from rucio.client.baseclient import BaseClient
 from rucio.client.baseclient import choice
@@ -26,8 +41,10 @@ class RSEClient(BaseClient):
 
     RSE_BASEURL = 'rses'
 
-    def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None, auth_type=None, creds=None, timeout=None):
-        super(RSEClient, self).__init__(rucio_host, auth_host, account, ca_cert, auth_type, creds, timeout)
+    def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None,
+                 auth_type=None, creds=None, timeout=600, user_agent='rucio-clients'):
+        super(RSEClient, self).__init__(rucio_host, auth_host, account, ca_cert,
+                                        auth_type, creds, timeout, user_agent)
 
     def get_rse(self, rse):
         """
@@ -47,7 +64,7 @@ class RSEClient(BaseClient):
             rse = loads(r.text)
             return rse
         else:
-            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def add_rse(self, rse, **kwargs):
@@ -64,6 +81,11 @@ class RSEClient(BaseClient):
         :param time_zone: Timezone.
         :param staging_area: Staging area.
         :param ISP: Internet service provider.
+        :param rse_type: RSE type.
+        :param latitude: Latitude coordinate of RSE.
+        :param longitude: Longitude coordinate of RSE.
+        :param ASN: Access service network.
+        :param availability: Availability.
 
         :return: True if location was created successfully else False.
         :raises Duplicate: if rse already exists.
@@ -73,7 +95,7 @@ class RSEClient(BaseClient):
         r = self._send_request(url, type='POST', data=dumps(kwargs))
         if r.status_code == codes.created:
             return True
-        exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
+        exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
         raise exc_cls(exc_msg)
 
     def update_rse(self, rse, parameters):
@@ -88,7 +110,7 @@ class RSEClient(BaseClient):
         r = self._send_request(url, type='PUT', data=dumps(parameters))
         if r.status_code == codes.created:
             return True
-        exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
+        exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
         raise exc_cls(exc_msg)
 
     def delete_rse(self, rse):
@@ -104,7 +126,7 @@ class RSEClient(BaseClient):
         if r.status_code == codes.ok:
             return True
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def list_rses(self, rse_expression=None):
@@ -115,7 +137,7 @@ class RSEClient(BaseClient):
         :return:         a list containing the names of all rucio locations.
         """
         if rse_expression:
-            path = ['rses/', "?expression=" + quote(rse_expression)]
+            path = ['rses', "?expression=" + quote(rse_expression)]
             path = '/'.join(path)
         else:
             path = 'rses/'
@@ -124,7 +146,7 @@ class RSEClient(BaseClient):
         if r.status_code == codes.ok:
             return self._load_json_data(r)
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def add_rse_attribute(self, rse, key, value):
@@ -146,7 +168,7 @@ class RSEClient(BaseClient):
         if r.status_code == codes.created:
             return True
         else:
-            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def delete_rse_attribute(self, rse, key):
@@ -165,7 +187,7 @@ class RSEClient(BaseClient):
         if r.status_code == codes.ok:
             return True
         else:
-            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def list_rse_attributes(self, rse):
@@ -174,7 +196,7 @@ class RSEClient(BaseClient):
 
         :param rse: The RSE name.
 
-        :return: True if RSE attribute was created successfully else False.
+        :return: A ``dict`` with the RSE attribute name/value pairs.
         """
         path = '/'.join([self.RSE_BASEURL, rse, 'attr/'])
         url = build_url(choice(self.list_hosts), path=path)
@@ -183,7 +205,7 @@ class RSEClient(BaseClient):
             attributes = loads(r.text)
             return attributes
         else:
-            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def add_protocol(self, rse, params):
@@ -195,19 +217,12 @@ class RSEClient(BaseClient):
         :param params: Attributes of the protocol. Supported are:
             hostname:       hostname for this protocol (default = localhost)
             port:           port for this protocol (default = 0)
-            prefix:         string used as a prfeix for this protocol when generating
-                            the PFN (default = None)
-            impl:           qualified name of the implementation class for this
-                            protocol (mandatory)
-            read:           integer representing the priority of this procotol for
-                            read operations (default = -1)
-            write:          integer representing the priority of this procotol for
-                            write operations (default = -1)
-            delete:         integer representing the priority of this procotol for
-                            delet operations (default = -1)
-            extended_attributes:  miscellaneous protocol specific information e.g. spacetoken
-                            for SRM (default = None)
-
+            prefix:         string used as a prfeix for this protocol when generating the PFN (default = None)
+            impl:           qualified name of the implementation class for this protocol (mandatory)
+            read:           integer representing the priority of this procotol for read operations (default = -1)
+            write:          integer representing the priority of this procotol for write operations (default = -1)
+            delete:         integer representing the priority of this procotol for delete operations (default = -1)
+            extended_attributes:  miscellaneous protocol specific information e.g. spacetoken for SRM (default = None)
 
         :return: True if protocol was created successfully else False.
 
@@ -225,7 +240,7 @@ class RSEClient(BaseClient):
         if r.status_code == codes.created:
             return True
         else:
-            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def get_protocols(self, rse, protocol_domain='ALL', operation=None, default=False, scheme=None):
@@ -266,7 +281,46 @@ class RSEClient(BaseClient):
             protocols = loads(r.text)
             return protocols
         else:
-            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
+            raise exc_cls(exc_msg)
+
+    def lfns2pfns(self, rse, lfns, protocol_domain='ALL', operation=None, scheme=None):
+        """
+        Returns PFNs that should be used at a RSE, corresponding to requested LFNs.
+        The PFNs are generated for the RSE *regardless* of whether a replica exists for the LFN.
+
+        :param rse: the RSE name
+        :param lfns: A list of LFN strings to translate to PFNs.
+        :param protocol_domain: The scope of the protocol. Supported are 'LAN', 'WAN', and 'ALL' (as default).
+        :param operation: The name of the requested operation (read, write, or delete).
+                          If None, all operations are queried.
+        :param scheme: The identifier of the requested protocol (gsiftp, https, davs, etc).
+
+        :returns: A dictionary of LFN / PFN pairs.
+        :raises RSENotFound: if the RSE doesn't exist.
+        :raises RSEProtocolNotSupported: if no matching protocol entry could be found.
+        :raises RSEOperationNotSupported: if no matching protocol entry for the requested
+                                          operation could be found.
+        """
+        path = '/'.join([self.RSE_BASEURL, rse, 'lfns2pfns'])
+        params = []
+        if scheme:
+            params.append(('scheme', scheme))
+        if protocol_domain != 'ALL':
+            params.append(('domain', protocol_domain))
+        if operation:
+            params.append(('operation', operation))
+        for lfn in lfns:
+            params.append(('lfn', lfn))
+
+        url = build_url(choice(self.list_hosts), path=path, params=params, doseq=True)
+
+        r = self._send_request(url, type='GET')
+        if r.status_code == codes.ok:
+            pfns = loads(r.text)
+            return pfns
+        else:
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def delete_protocols(self, rse, scheme, hostname=None, port=None):
@@ -297,7 +351,7 @@ class RSEClient(BaseClient):
         if r.status_code == codes.ok:
             return True
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def update_protocols(self, rse, scheme, data, hostname=None, port=None):
@@ -331,9 +385,41 @@ class RSEClient(BaseClient):
         if r.status_code == codes.ok:
             return True
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
+    def swap_protocols(self, rse, domain, operation, scheme_a, scheme_b):
+        """
+        Swaps the priorities of the provided operation.
+
+        :param rse: the RSE name.
+        :param domain: the domain in which priorities should be swapped i.e. wan or lan.
+        :param operation: the operation that should be swapped i.e. read, write, or delete.
+        :param scheme_a: the scheme of one of the two protocols to be swapped, e.g. srm.
+        :param scheme_b: the scheme of the other of the two protocols to be swapped, e.g. http.
+
+        :returns: True if success.
+
+        :raises RSEProtocolNotSupported: if no matching protocol entry could be found.
+        :raises RSENotFound: if the RSE doesn't exist.
+        :raises KeyNotFound: if invalid data was provided for update.
+        :raises AccessDenied: if not authorized.
+        """
+        protocol_a = protocol_b = None
+        protocols = self.get_protocols(rse, domain, operation, False, scheme_a)['protocols']
+        for p in protocols:
+            if p['scheme'] == scheme_a:
+                protocol_a = p
+            if p['scheme'] == scheme_b:
+                protocol_b = p
+        if (protocol_a or protocol_b) is None:
+            return False
+        priority_a = protocol_a['domains'][domain][operation]
+        priority_b = protocol_b['domains'][domain][operation]
+        self.update_protocols(rse, protocol_a['scheme'], {'domains': {domain: {operation: priority_b}}}, protocol_a['hostname'], protocol_a['port'])
+        self.update_protocols(rse, protocol_b['scheme'], {'domains': {domain: {operation: priority_a}}}, protocol_b['hostname'], protocol_b['port'])
+        return True
+
     def set_rse_usage(self, rse, source, used, free):
         """
         Set RSE usage information.
@@ -353,7 +439,7 @@ class RSEClient(BaseClient):
         if r.status_code == codes.ok:
             return True
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def get_rse_usage(self, rse, filters=None):
@@ -372,7 +458,7 @@ class RSEClient(BaseClient):
         if r.status_code == codes.ok:
             return self._load_json_data(r)
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
     def list_rse_usage_history(self, rse, filters=None):
@@ -391,7 +477,9 @@ class RSEClient(BaseClient):
         if r.status_code == codes.ok:
             return self._load_json_data(r)
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers,
+                                                   status_code=r.status_code,
+                                                   data=r.content)
             raise exc_cls(exc_msg)
 
     def set_rse_limits(self, rse, name, value):
@@ -408,11 +496,11 @@ class RSEClient(BaseClient):
         path = '/'.join(path)
         url = build_url(choice(self.list_hosts), path=path)
         r = self._send_request(url, type='PUT', data=dumps({'name': name, 'value': value}))
-
         if r.status_code == codes.ok:
             return True
-
-        exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+        exc_cls, exc_msg = self._get_exception(headers=r.headers,
+                                               status_code=r.status_code,
+                                               data=r.content)
         raise exc_cls(exc_msg)
 
     def get_rse_limits(self, rse):
@@ -423,13 +511,69 @@ class RSEClient(BaseClient):
 
         :returns: True if successful, otherwise false.
         """
-
         path = [self.RSE_BASEURL, rse, 'limits']
         path = '/'.join(path)
         url = build_url(choice(self.list_hosts), path=path)
         r = self._send_request(url, type='GET')
         if r.status_code == codes.ok:
-            return self._load_json_data(r).next()
-        else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
-            raise exc_cls(exc_msg)
+            return next(self._load_json_data(r))
+        exc_cls, exc_msg = self._get_exception(headers=r.headers,
+                                               status_code=r.status_code,
+                                               data=r.content)
+        raise exc_cls(exc_msg)
+
+    def add_distance(self, source, destination, parameters):
+        """
+        Add a src-dest distance.
+
+        :param source: The source.
+        :param destination: The destination.
+        :param parameters: A dictionnary with property.
+        """
+        path = [self.RSE_BASEURL, source, 'distances', destination]
+        path = '/'.join(path)
+        url = build_url(choice(self.list_hosts), path=path)
+        r = self._send_request(url, type='POST', data=dumps(parameters))
+        if r.status_code == codes.created:
+            return True
+        exc_cls, exc_msg = self._get_exception(headers=r.headers,
+                                               status_code=r.status_code,
+                                               data=r.content)
+        raise exc_cls(exc_msg)
+
+    def update_distance(self, source, destination, parameters):
+        """
+        Update distances with the given RSE ids.
+
+        :param source: The source.
+        :param destination: The destination.
+        :param parameters: A dictionnary with property.
+        """
+        path = [self.RSE_BASEURL, source, 'distances', destination]
+        path = '/'.join(path)
+        url = build_url(choice(self.list_hosts), path=path)
+        r = self._send_request(url, type='PUT', data=dumps(parameters))
+        if r.status_code == codes.ok:
+            return True
+        exc_cls, exc_msg = self._get_exception(headers=r.headers,
+                                               status_code=r.status_code,
+                                               data=r.content)
+        raise exc_cls(exc_msg)
+
+    def get_distance(self, source, destination):
+        """
+        Get distances between rses.
+
+        :param source: The source RSE.
+        :param destination: The destination RSE.
+
+        :returns distance: List of dictionaries.
+        """
+        path = [self.RSE_BASEURL, source, 'distances', destination]
+        path = '/'.join(path)
+        url = build_url(choice(self.list_hosts), path=path)
+        r = self._send_request(url, type='GET')
+        if r.status_code == codes.ok:
+            return next(self._load_json_data(r))
+        exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
+        raise exc_cls(exc_msg)
