@@ -1,11 +1,24 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
-# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2013-2014
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2018
+# - Ralph Vigne <ralph.vigne@cern.ch>, 2015
+# - Vincent Garonne <vgaronne@gmail.com>, 2015-2018
+#
+# PY3K COMPATIBLE
 
 from json import dumps
 from requests.status_codes import codes
@@ -21,10 +34,10 @@ class SubscriptionClient(BaseClient):
 
     SUB_BASEURL = 'subscriptions'
 
-    def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None, auth_type=None, creds=None, timeout=None):
-        super(SubscriptionClient, self).__init__(rucio_host, auth_host, account, ca_cert, auth_type, creds, timeout)
+    def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None, auth_type=None, creds=None, timeout=600, user_agent='rucio-clients'):
+        super(SubscriptionClient, self).__init__(rucio_host, auth_host, account, ca_cert, auth_type, creds, timeout, user_agent)
 
-    def add_subscription(self, name, account, filter, replication_rules, comments, lifetime, retroactive, dry_run):
+    def add_subscription(self, name, account, filter, replication_rules, comments, lifetime, retroactive, dry_run, priority=3):
         """
         Adds a new subscription which will be verified against every new added file and dataset
 
@@ -45,20 +58,24 @@ class SubscriptionClient(BaseClient):
         :type retroactive:  Boolean
         :param dry_run: Just print the subscriptions actions without actually executing them (Useful if retroactive flag is set)
         :type dry_run:  Boolean
+        :param priority: The priority of the subscription (3 by default)
+        :type priority: Integer
         """
         path = self.SUB_BASEURL + '/' + account + '/' + name
         url = build_url(choice(self.list_hosts), path=path)
-        if filter and type(filter) != dict:
+        if retroactive:
+            raise NotImplementedError('Retroactive mode is not implemented')
+        if filter and not isinstance(filter, dict):
             raise TypeError('filter should be a dict')
-        if replication_rules and type(replication_rules) != list:
+        if replication_rules and not isinstance(replication_rules, list):
             raise TypeError('replication_rules should be a list')
-        data = dumps({'filter': filter, 'replication_rules': replication_rules, 'comments': comments,
-                      'lifetime': lifetime, 'retroactive': retroactive, 'dry_run': dry_run})
-        r = self._send_request(url, type='POST', data=data)
-        if r.status_code == codes.created:
-            return r.text
+        data = dumps({'options': {'filter': filter, 'replication_rules': replication_rules, 'comments': comments,
+                                  'lifetime': lifetime, 'retroactive': retroactive, 'dry_run': dry_run, 'priority': priority}})
+        result = self._send_request(url, type='POST', data=data)
+        if result.status_code == codes.created:   # pylint: disable=no-member
+            return result.text
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=result.headers, status_code=result.status_code, data=result.content)
             raise exc_cls(exc_msg)
 
     def list_subscriptions(self, name=None, account=None):
@@ -82,14 +99,14 @@ class SubscriptionClient(BaseClient):
         else:
             path += '/'
         url = build_url(choice(self.list_hosts), path=path)
-        r = self._send_request(url, type='GET')
-        if r.status_code == codes.ok:
-            return self._load_json_data(r)
+        result = self._send_request(url, type='GET')
+        if result.status_code == codes.ok:   # pylint: disable=no-member
+            return self._load_json_data(result)
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=result.headers, status_code=result.status_code, data=result.content)
             raise exc_cls(exc_msg)
 
-    def update_subscription(self, name, account=None, filter=None, replication_rules=None, comments=None, lifetime=None, retroactive=None, dry_run=None):
+    def update_subscription(self, name, account=None, filter=None, replication_rules=None, comments=None, lifetime=None, retroactive=None, dry_run=None, priority=None):
         """
         Updates a subscription
 
@@ -110,23 +127,27 @@ class SubscriptionClient(BaseClient):
         :type retroactive:  Boolean
         :param dry_run: Just print the subscriptions actions without actually executing them (Useful if retroactive flag is set)
         :type dry_run:  Boolean
+        :param priority: The priority of the subscription
+        :type priority: Integer
         :raises: exception.NotFound if subscription is not found
         """
         if not account:
             account = self.account
+        if retroactive:
+            raise NotImplementedError('Retroactive mode is not implemented')
         path = self.SUB_BASEURL + '/' + account + '/' + name
         url = build_url(choice(self.list_hosts), path=path)
-        if filter and type(filter) != dict:
+        if filter and not isinstance(filter, dict):
             raise TypeError('filter should be a dict')
-        if replication_rules and type(replication_rules) != list:
+        if replication_rules and not isinstance(replication_rules, list):
             raise TypeError('replication_rules should be a list')
-        data = dumps({'filter': filter, 'replication_rules': replication_rules, 'comments': comments,
-                      'lifetime': lifetime, 'retroactive': retroactive, 'dry_run': dry_run})
-        r = self._send_request(url, type='PUT', data=data)
-        if r.status_code == codes.created:
+        data = dumps({'options': {'filter': filter, 'replication_rules': replication_rules, 'comments': comments,
+                                  'lifetime': lifetime, 'retroactive': retroactive, 'dry_run': dry_run, 'priority': priority}})
+        result = self._send_request(url, type='PUT', data=data)
+        if result.status_code == codes.created:   # pylint: disable=no-member
             return True
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=result.headers, status_code=result.status_code, data=result.content)
             raise exc_cls(exc_msg)
 
     def list_subscription_rules(self, account, name):
@@ -139,9 +160,9 @@ class SubscriptionClient(BaseClient):
 
         path = '/'.join([self.SUB_BASEURL, account, name, 'Rules'])
         url = build_url(choice(self.list_hosts), path=path)
-        r = self._send_request(url, type='GET')
-        if r.status_code == codes.ok:
-            return self._load_json_data(r)
+        result = self._send_request(url, type='GET')
+        if result.status_code == codes.ok:   # pylint: disable=no-member
+            return self._load_json_data(result)
         else:
-            exc_cls, exc_msg = self._get_exception(r.headers, r.status_code)
+            exc_cls, exc_msg = self._get_exception(headers=result.headers, status_code=result.status_code, data=result.content)
             raise exc_cls(exc_msg)
