@@ -1,14 +1,27 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
+# you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# http://www.apache.org/licenses/LICENSE-2.0
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Ralph Vigne, <ralph.vigne@cern.ch>, 2012-214
-# - Yun-Pin Sun, <yun-pin.sun@cern.ch>, 2012
-
+# - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2014
+# - Vincent Garonne <vgaronne@gmail.com>, 2012-2018
+# - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
+# - Yun-Pin Sun <winter0128@gmail.com>, 2012
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2013
+# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2018
+# - Nicolo Magini, <nicolo.magini@cern.ch>, 2018
+#
+# PY3K COMPATIBLE
 
 import pysftp
 from subprocess import call
@@ -60,12 +73,13 @@ class Default(protocol.RSEProtocol):
         """ Closes the connection to RSE."""
         self.__connection.close()
 
-    def get(self, pfn, dest):
+    def get(self, pfn, dest, transfer_timeout=None):
         """
             Provides access to files stored inside connected the RSE.
 
             :param path: Physical file name of requested file
             :param dest: Name and path of the files when stored at the client
+            :param transfer_timeout: Transfer timeout (in seconds) - dummy
 
             :raises DestinationNotAccessible: if the destination storage was not accessible.
             :raises ServiceUnavailable: if some generic error occured in the library.
@@ -88,13 +102,14 @@ class Default(protocol.RSEProtocol):
             else:
                 raise exception.ServiceUnavailable(e)
 
-    def put(self, source, target, source_dir=None):
+    def put(self, source, target, source_dir=None, transfer_timeout=None):
         """
             Allows to store files inside the referred RSE.
 
             :param source: path to the source file on the client file system
             :param target: path to the destination file on the storage
             :param source_dir: Path where the to be transferred files are stored in the local file system
+            :param transfer_timeout: Transfer timeout (in seconds) - dummy
 
             :raises DestinationNotAccessible: if the destination storage was not accessible.
             :raises ServiceUnavailable: if some generic error occured in the library.
@@ -107,11 +122,11 @@ class Default(protocol.RSEProtocol):
         try:
             self.__connection.put(sf, self.pfn2path(target))
         except IOError as e:
-                try:
-                    self.__connection.execute('mkdir -p %s' % '/'.join(self.pfn2path(target).split('/')[0:-1]))
-                    self.__connection.put(sf, self.pfn2path(target))
-                except Exception, e:
-                    raise exception.DestinationNotAccessible(e)
+            try:
+                self.__connection.execute('mkdir -p %s' % '/'.join(self.pfn2path(target).split('/')[0:-1]))
+                self.__connection.put(sf, self.pfn2path(target))
+            except Exception as e:
+                raise exception.DestinationNotAccessible(e)
         except OSError as e:
             if e.errno == 2:
                 raise exception.SourceNotFound(e)
@@ -162,7 +177,7 @@ class Default(protocol.RSEProtocol):
                 raise exception.ServiceUnavailable(e)
 
     def pfn2path(self, pfn):
-        tmp = self.parse_pfns(pfn).values()[0]
+        tmp = list(self.parse_pfns(pfn).values())[0]
         return '/'.join([tmp['prefix'], tmp['path'], tmp['name']])
 
     def stat(self, pfn):
