@@ -7,14 +7,30 @@
 #
 # Authors:
 # - Vincent Garonne, <vincent.garonne@cern.ch>, 2012-2013
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2013-2014
+# - Cedric Serfon, <cedric.serfon@cern.ch>, 2013-2015
 # - Wen Guan, <wguan@cern.ch>, 2014
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2014
+# - Mario Lassnig, <mario.lassnig@cern.ch>, 2014, 2017
+# - Nicolo Magini, <nicolo.magini@cern.ch>, 2018
+# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
+#
+# PY3K COMPATIBLE
 
-import commands
 import os
 import re
-import urlparse
+try:
+    # PY2
+    import urlparse
+except ImportError:
+    # PY3
+    import urllib.parse as urlparse
+
+try:
+    # PY2
+    from commands import getstatusoutput
+except ImportError:
+    # PY3
+    from subprocess import getstatusoutput
+from six import string_types
 
 from rucio.common import exception
 from rucio.common.utils import execute
@@ -35,7 +51,7 @@ class Default(protocol.RSEProtocol):
         pfns = {}
         prefix = self.attributes['prefix']
         if self.attributes['extended_attributes'] is not None and\
-           'web_service_path' in self.attributes['extended_attributes'].keys():
+           'web_service_path' in list(self.attributes['extended_attributes'].keys()):
             web_service_path = self.attributes['extended_attributes']['web_service_path']
         else:
             web_service_path = ''
@@ -52,16 +68,20 @@ class Default(protocol.RSEProtocol):
         lfns = [lfns] if type(lfns) == dict else lfns
         if not self.attributes['port']:
             for lfn in lfns:
-                scope, name, path = lfn['scope'], lfn['name'], lfn.get('path')
+                scope, name, path = str(lfn['scope']), lfn['name'], lfn.get('path')
                 if not path:
                     path = self._get_path(scope=scope, name=name)
+                if path.startswith('/'):
+                    path = path[1:]
                 pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'], '://',
                                                          hostname, web_service_path, prefix, path])
         else:
             for lfn in lfns:
-                scope, name, path = lfn['scope'], lfn['name'], lfn.get('path')
+                scope, name, path = str(lfn['scope']), lfn['name'], lfn.get('path')
                 if not path:
                     path = self._get_path(scope=scope, name=name)
+                if path.startswith('/'):
+                    path = path[1:]
                 pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'], '://',
                                                          hostname, ':', str(self.attributes['port']),
                                                          web_service_path, prefix, path])
@@ -79,7 +99,7 @@ class Default(protocol.RSEProtocol):
         """
 
         ret = dict()
-        pfns = [pfns] if ((type(pfns) == str) or (type(pfns) == unicode)) else pfns
+        pfns = [pfns] if isinstance(pfns, string_types) else pfns
         for pfn in pfns:
             parsed = urlparse.urlparse(pfn)
             if parsed.path.startswith('/srm/managerv2') or\
@@ -93,6 +113,12 @@ class Default(protocol.RSEProtocol):
                 path = parsed.path
                 service_path = ''
 
+            # force type conversion
+            try:
+                port = int(port)
+            except:
+                port = ''
+
             if self.attributes['hostname'] != hostname and\
                self.attributes['hostname'] != scheme + "://" + hostname:
                 raise exception.RSEFileNameNotSupported('Invalid hostname: provided \'%s\', expected \'%s\'' % (hostname,
@@ -112,7 +138,11 @@ class Default(protocol.RSEProtocol):
             prefix = self.attributes['prefix']
             path = path.partition(self.attributes['prefix'])[2]
             name = path.split('/')[-1]
-            path = path.partition(name)[0]
+            path = '/' + '/'.join(path.split('/')[:-1]) if not self.rse['staging_area'] else None
+
+            if path != '/' and path[:-1] != '/':
+                path += '/'
+
             ret[pfn] = {'scheme': scheme, 'port': port, 'hostname': hostname,
                         'path': path, 'name': name, 'prefix': prefix,
                         'web_service_path': service_path}
@@ -134,9 +164,9 @@ class Default(protocol.RSEProtocol):
         if '://' in hostname:
             hostname = hostname.split("://")[1]
 
-        if 'extended_attributes' in self.attributes.keys() and\
+        if 'extended_attributes' in list(self.attributes.keys()) and\
            self.attributes['extended_attributes'] is not None and\
-           'web_service_path' in self.attributes['extended_attributes'].keys():
+           'web_service_path' in list(self.attributes['extended_attributes'].keys()):
             web_service_path = self.attributes['extended_attributes']['web_service_path']
         else:
             web_service_path = ''
@@ -158,48 +188,55 @@ class Default(protocol.RSEProtocol):
         :raises RSEAccessDenied: Cannot connect.
         """
 
-        status, lcglscommand = commands.getstatusoutput('which lcg-ls')
+        status, lcglscommand = getstatusoutput('which lcg-ls')
         if status:
             raise exception.RSEAccessDenied('Cannot find lcg tools')
         endpoint_basepath = self.path2pfn(self.attributes['prefix'])
-        status, result = commands.getstatusoutput('%s $LCGVO -b --srm-timeout 60 -D srmv2 -l %s' % (lcglscommand, endpoint_basepath))
+        status, result = getstatusoutput('%s -vv $LCGVO -b --srm-timeout 60 -D srmv2 -l %s' % (lcglscommand, endpoint_basepath))
         if status:
-            raise exception.RSEAccessDenied('Endpoint not reachable')
+            if result == '':
+                raise exception.RSEAccessDenied('Endpoint not reachable. lcg-ls failed with status code %s but no further details.' % (str(status)))
+            else:
+                raise exception.RSEAccessDenied('Endpoint not reachable : %s' % str(result))
 
-    def get(self, path, dest):
+    def get(self, path, dest, transfer_timeout=None):
         """
         Provides access to files stored inside connected the RSE.
 
         :param path: Physical file name of requested file
         :param dest: Name and path of the files when stored at the client
+        :param transfer_timeout: Transfer timeout (in seconds)
+
         :raises DestinationNotAccessible: if the destination storage was not accessible.
         :raises ServiceUnavailable: if some generic error occured in the library.
         :raises SourceNotFound: if the source file was not found on the referred storage.
         """
 
-        space_token = ''
-        if self.attributes['extended_attributes'] is not None and 'space_token' in self.attributes['extended_attributes'].keys():
-            space_token = '--sst %s' % self.attributes['extended_attributes']['space_token']
+        timeout_option = ''
+        if transfer_timeout:
+            timeout_option = '--sendreceive-timeout %s' % transfer_timeout
 
         try:
-            cmd = 'lcg-cp $LCGVO -v -b --srm-timeout 3600 -D srmv2 %s %s file:%s' % (space_token, path, dest)
+            cmd = 'lcg-cp $LCGVO -v -b --srm-timeout 3600 %s -D srmv2 %s file:%s' % (timeout_option, path, dest)
             status, out, err = execute(cmd)
             if status:
                 if self.__parse_srm_error__("SRM_INVALID_PATH", out, err):
                     raise exception.SourceNotFound(err)
                 raise exception.RucioException(err)
-        except exception.SourceNotFound as e:
-            raise exception.SourceNotFound(str(e))
-        except Exception as e:
-            raise exception.ServiceUnavailable(e)
+        except exception.SourceNotFound as error:
+            raise exception.SourceNotFound(str(error))
+        except Exception as error:
+            raise exception.ServiceUnavailable(error)
 
-    def put(self, source, target, source_dir):
+    def put(self, source, target, source_dir, transfer_timeout=None):
         """
         Allows to store files inside the referred RSE.
 
         :param source: path to the source file on the client file system
         :param target: path to the destination file on the storage
         :param source_dir: Path where the to be transferred files are stored in the local file system
+        :param transfer_timeout: Transfer timeout (in seconds)
+
         :raises DestinationNotAccessible: if the destination storage was not accessible.
         :raises ServiceUnavailable: if some generic error occured in the library.
         :raises SourceNotFound: if the source file was not found on the referred storage.
@@ -211,16 +248,20 @@ class Default(protocol.RSEProtocol):
             raise exception.SourceNotFound()
 
         space_token = ''
-        if self.attributes['extended_attributes'] is not None and 'space_token' in self.attributes['extended_attributes'].keys():
+        if self.attributes['extended_attributes'] is not None and 'space_token' in list(self.attributes['extended_attributes'].keys()):
             space_token = '--dst %s' % self.attributes['extended_attributes']['space_token']
 
+        timeout_option = ''
+        if transfer_timeout:
+            timeout_option = '--sendreceive-timeout %s' % transfer_timeout
+
         try:
-            cmd = 'lcg-cp $LCGVO -v -b --srm-timeout 3600 -D srmv2 %s file:%s %s' % (space_token, source_url, target)
+            cmd = 'lcg-cp $LCGVO -v -b --srm-timeout 3600 %s -D srmv2 %s file:%s %s' % (timeout_option, space_token, source_url, target)
             status, out, err = execute(cmd)
             if status:
                 raise exception.RucioException(err)
-        except Exception as e:
-            raise exception.ServiceUnavailable(e)
+        except Exception as error:
+            raise exception.ServiceUnavailable(error)
 
     def delete(self, path):
         """
@@ -231,7 +272,7 @@ class Default(protocol.RSEProtocol):
         :raises SourceNotFound: if the source file was not found on the referred storage.
         """
 
-        pfns = [path] if ((type(path) == str) or (type(path) == unicode)) else path
+        pfns = [path] if isinstance(path, string_types) else path
 
         try:
             pfn_chunks = [pfns[i:i + 20] for i in range(0, len(pfns), 20)]
@@ -244,10 +285,10 @@ class Default(protocol.RSEProtocol):
                     if self.__parse_srm_error__("SRM_INVALID_PATH", out, err):
                         raise exception.SourceNotFound(err)
                     raise exception.RucioException(err)
-        except exception.SourceNotFound as e:
-            raise exception.SourceNotFound(str(e))
-        except Exception as e:
-            raise exception.ServiceUnavailable(e)
+        except exception.SourceNotFound as error:
+            raise exception.SourceNotFound(str(error))
+        except Exception as error:
+            raise exception.ServiceUnavailable(error)
 
     def rename(self, path, new_path):
         """
@@ -260,8 +301,12 @@ class Default(protocol.RSEProtocol):
         :raises SourceNotFound: if the source file was not found on the referred storage.
         """
 
+        space_token = ''
+        if self.attributes['extended_attributes'] is not None and 'space_token' in list(self.attributes['extended_attributes'].keys()):
+            space_token = '--dst %s' % self.attributes['extended_attributes']['space_token']
+
         try:
-            cmd = 'lcg-cp $LCGVO -v -b --srm-timeout 3600 -D srmv2 %s %s' % (path, new_path)
+            cmd = 'lcg-cp $LCGVO -v -b --srm-timeout 3600 -D srmv2 %s %s %s' % (space_token, path, new_path)
             status, out, err = execute(cmd)
             if status:
                 raise exception.RucioException(err)
@@ -270,8 +315,8 @@ class Default(protocol.RSEProtocol):
             status, out, err = execute(cmd)
             if status:
                 raise exception.RucioException(err)
-        except Exception as e:
-            raise exception.ServiceUnavailable(e)
+        except Exception as error:
+            raise exception.ServiceUnavailable(error)
 
     def exists(self, path):
         """
@@ -288,8 +333,8 @@ class Default(protocol.RSEProtocol):
             if status:
                 return False
             return True
-        except Exception as e:
-            raise exception.ServiceUnavailable(e)
+        except Exception as error:
+            raise exception.ServiceUnavailable(error)
 
     def __parse_srm_error__(self, err_code, out, err):
         """Parse the error message to return error code."""
