@@ -1,17 +1,34 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
+# you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# http://www.apache.org/licenses/LICENSE-2.0
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Ralph Vigne, <ralph.vigne@cern.ch>, 2012-2014
-# - Wen Guan, <wguan@cern.ch>, 2014
+# - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2014
+# - Vincent Garonne <vgaronne@gmail.com>, 2012-2018
+# - Wen Guan <wguan.icedew@gmail.com>, 2014-2016
+# - Martin Barisits <martin.barisits@cern.ch>, 2016
+# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2018
+
+from __future__ import print_function
 
 import json
 import os
 import os.path
+import tempfile
+try:
+    from exceptions import NotImplementedError
+except ImportError:
+    pass
 
 from rucio.common.utils import adler32
 from rucio.rse import rsemanager as mgr
@@ -34,9 +51,10 @@ class MgrTestCases():
                 data = json.load(f)
             self.rse_settings['credentials'] = data[rse_tag]
         except KeyError:
-            print 'No credentials found for this RSE.'
+            print('No credentials found for this RSE.')
             pass
         self.tmpdir = tmpdir
+        self.gettmpdir = tempfile.mkdtemp()
         self.user = user
         self.static_file = static_file
 
@@ -56,17 +74,17 @@ class MgrTestCases():
                                                            {'name': '2_rse_remote_get.raw', 'scope': 'user.%s' % self.user},
                                                            {'name': '3_rse_remote_get.raw', 'scope': 'user.%s' % self.user, 'pfn': self.static_file},
                                                            {'name': '4_rse_remote_get.raw', 'scope': 'user.%s' % self.user, 'pfn': pfn_b}],
-                                       self.tmpdir)
+                                       self.gettmpdir)
         if not (status and details['user.%s:1_rse_remote_get.raw' % self.user] and details['user.%s:2_rse_remote_get.raw' % self.user] and details['user.%s:3_rse_remote_get.raw' % self.user] and details['user.%s:4_rse_remote_get.raw' % self.user]):
             raise Exception('Return not as expected: %s, %s' % (status, details))
 
     def test_get_mgr_ok_single_lfn(self):
         """(RSE/PROTOCOLS): Get a single file from storage provding the LFN (Success)"""
-        mgr.download(self.rse_settings, {'name': '1_rse_remote_get.raw', 'scope': 'user.%s' % self.user}, self.tmpdir)
+        mgr.download(self.rse_settings, {'name': '1_rse_remote_get.raw', 'scope': 'user.%s' % self.user}, self.gettmpdir)
 
     def test_get_mgr_ok_single_pfn(self):
         """(RSE/PROTOCOLS): Get a single file from storage providing the PFN (Success)"""
-        mgr.download(self.rse_settings, {'name': '1_rse_remote_get.raw', 'scope': 'user.%s' % self.user, 'pfn': self.static_file}, self.tmpdir)
+        mgr.download(self.rse_settings, {'name': '1_rse_remote_get.raw', 'scope': 'user.%s' % self.user, 'pfn': self.static_file}, self.gettmpdir)
 
     def test_get_mgr_SourceNotFound_multi(self):
         """(RSE/PROTOCOLS): Get multiple files from storage providing LFNs  and PFNs (SourceNotFound)"""
@@ -76,7 +94,7 @@ class MgrTestCases():
         status, details = mgr.download(self.rse_settings, [{'name': '1_not_existing_data.raw', 'scope': 'user.%s' % self.user},
                                                            {'name': '1_rse_remote_get.raw', 'scope': 'user.%s' % self.user},
                                                            {'name': '2_not_existing_data.raw', 'scope': 'user.%s' % self.user, 'pfn': pfn_b},
-                                                           {'name': '2_rse_remote_get.raw', 'scope': 'user.%s' % self.user, 'pfn': pfn_a}], self.tmpdir)
+                                                           {'name': '2_rse_remote_get.raw', 'scope': 'user.%s' % self.user, 'pfn': pfn_a}], self.gettmpdir)
         if details['user.%s:1_rse_remote_get.raw' % self.user] and details['user.%s:2_rse_remote_get.raw' % self.user]:
             if details['user.%s:1_not_existing_data.raw' % self.user].__class__.__name__ == 'SourceNotFound' and details['user.%s:2_not_existing_data.raw' % self.user].__class__.__name__ == 'SourceNotFound':
                 raise details['user.%s:1_not_existing_data.raw' % self.user]
@@ -87,12 +105,12 @@ class MgrTestCases():
 
     def test_get_mgr_SourceNotFound_single_lfn(self):
         """(RSE/PROTOCOLS): Get a single file from storage providing LFN (SourceNot Found)"""
-        mgr.download(self.rse_settings, {'name': 'not_existing_data.raw', 'scope': 'user.%s' % self.user}, self.tmpdir)
+        mgr.download(self.rse_settings, {'name': 'not_existing_data.raw', 'scope': 'user.%s' % self.user}, self.gettmpdir)
 
     def test_get_mgr_SourceNotFound_single_pfn(self):
         """(RSE/PROTOCOLS): Get a single file from storage providing PFN (SourceNotF ound)"""
         pfn = mgr.lfns2pfns(self.rse_settings, {'name': 'not_existing_data.raw', 'scope': 'user.%s' % self.user}).values()[0]
-        mgr.download(self.rse_settings, {'name': 'not_existing_data.raw', 'scope': 'user.%s' % self.user, 'pfn': pfn}, self.tmpdir)
+        mgr.download(self.rse_settings, {'name': 'not_existing_data.raw', 'scope': 'user.%s' % self.user, 'pfn': pfn}, self.gettmpdir)
 
     # Mgr-Tests: PUT
     def test_put_mgr_ok_multi(self):
@@ -145,12 +163,19 @@ class MgrTestCases():
         """(RSE/PROTOCOLS): Delete multiple files from storage (Success)"""
         status, details = mgr.delete(self.rse_settings, [{'name': '1_rse_remote_delete.raw', 'scope': 'user.%s' % self.user}, {'name': '2_rse_remote_delete.raw', 'scope': 'user.%s' % self.user}])
         if not (status and details['user.%s:1_rse_remote_delete.raw' % self.user] and details['user.%s:2_rse_remote_delete.raw' % self.user]):
-            raise Exception('Return not as expected: %s, %s' % (status, details))
+            if isinstance(details['user.%s:1_rse_remote_delete.raw' % self.user], NotImplementedError) and isinstance(details['user.%s:2_rse_remote_delete.raw' % self.user], NotImplementedError):
+                pass
+            else:
+                raise Exception('Return not as expected: %s, %s' % (status, details))
 
     def test_delete_mgr_ok_single(self):
         """(RSE/PROTOCOLS): Delete a single file from storage (Success)"""
         mgr.delete(self.rse_settings, {'name': '3_rse_remote_delete.raw', 'scope': 'user.%s' % self.user})
 
+    def test_delete_mgr_ok_dir(self):
+        """(RSE/PROTOCOLS): Delete a directory from storage (Success)"""
+        mgr.delete(self.rse_settings, {'path': 'user/%s' % self.user, 'name': 'user.%s' % self.user, 'scope': 'user.%s' % self.user})
+
     def test_delete_mgr_SourceNotFound_multi(self):
         """(RSE/PROTOCOLS): Delete multiple files from storage (SourceNotFound)"""
         status, details = mgr.delete(self.rse_settings, [{'name': 'not_existing_data.raw', 'scope': 'user.%s' % self.user}, {'name': '4_rse_remote_delete.raw', 'scope': 'user.%s' % self.user}])
@@ -239,7 +264,7 @@ class MgrTestCases():
                                                          {'name': '5_rse_remote_rename.raw', 'scope': 'user.%s' % self.user, 'new_name': '5_rse_new.raw'},
                                                          {'name': pfn_a, 'new_name': pfn_a_new},
                                                          {'name': pfn_b, 'new_name': pfn_b_new}])
-        if (not status and details['user.%s:5_rse_remote_rename.raw' % self.user] and details[pfn_b]) and (type(details['user.%s:4_rse_remote_rename.raw' % self.user]) == type(details[pfn_a])):
+        if (not status and details['user.%s:5_rse_remote_rename.raw' % self.user] and details[pfn_b]) and isinstance(details['user.%s:4_rse_remote_rename.raw' % self.user], type(details[pfn_a])):
             raise details['user.%s:4_rse_remote_rename.raw' % self.user]
         else:
             raise Exception('Return not as expected: %s, %s' % (status, details))
@@ -260,7 +285,7 @@ class MgrTestCases():
         pfn_b = mgr.lfns2pfns(self.rse_settings, {'name': '1_rse_not_created.raw', 'scope': 'user.%s' % self.user}).values()[0]
         status, details = mgr.rename(self.rse_settings, [{'name': '1_rse_not_existing.raw', 'scope': 'user.%s' % self.user, 'new_name': '1_rse_new_not_created.raw'},
                                                          {'name': pfn_a, 'new_name': pfn_b}])
-        if not status and (type(details['user.%s:1_rse_not_existing.raw' % self.user]) == type(details[pfn_a])):
+        if not status and isinstance(details['user.%s:1_rse_not_existing.raw' % self.user], type(details[pfn_a])):
             raise details['user.%s:1_rse_not_existing.raw' % self.user]
         else:
             raise Exception('Return not as expected: %s, %s' % (status, details))
