@@ -41,7 +41,7 @@ from rucio.common.exception import (Duplicate, RSENotFound, RSEProtocolNotSuppor
                                     InvalidObject, RSEProtocolDomainNotSupported, RSEProtocolPriorityError,
                                     ResourceTemporaryUnavailable, RSEAttributeNotFound, RSEOperationNotSupported)
 from rucio.common.utils import generate_uuid
-from rucio.core.rse import (add_rse, get_rse_id, del_rse, list_rses, rse_exists, add_rse_attribute, list_rse_attributes,
+from rucio.core.rse import (add_rse, get_rse_id, del_rse, restore_rse, list_rses, rse_exists, add_rse_attribute, list_rse_attributes,
                             set_rse_transfer_limits, get_rse_transfer_limits, delete_rse_transfer_limits,
                             get_rse_protocols, del_rse_attribute, get_rse_attribute, get_rse, rse_is_empty)
 from rucio.rse import rsemanager as mgr
@@ -180,6 +180,30 @@ class TestRSECoreApi(object):
         # with assert_raises(RSENotFound):
         #     del_rse(rse=rse_name)
 
+    def test_restore_rse(self):
+        """ RSE (CORE): Test restore of RSE """
+        # Restore deleted RSE
+        rse_name = rse_name_generator()
+        rse_id = add_rse(rse_name)
+        db_session = session.get_session()
+        db_session.commit()
+
+        del_rse(rse_id)
+        db_session.commit()
+        # Verify RSE was deleted
+        assert_equal(rse_exists(rse=rse_id), False)
+
+        restore_rse(rse_id=rse_id)
+        db_session.commit()
+        # Verify RSE was restored
+        assert rse_exists(rse=rse_name)
+
+        # Restoration of not deleted RSE:
+        rse_name = rse_name_generator()
+        rse_id = add_rse(rse_name)
+        with assert_raises(RSENotFound):
+            restore_rse(rse_id=rse_id)
+
     def test_empty_rse(self):
         """ RSE (CORE): Test if RSE is empty """
         rse_name = rse_name_generator()
@@ -405,13 +429,6 @@ class TestRSEClient(object):
 
     def test_update_rse(self):
         """ RSE (CLIENTS): update rse."""
-        # Check if updating RSE does not remove RSE tag
-        rse = rse_name_generator()
-        ret = self.client.add_rse(rse)
-        assert_equal(get_rse_attribute(key=rse, rse_id=get_rse_id(rse)), [True])
-        self.client.update_rse(rse, {'availability_write': False, 'availability_delete': False})
-        assert_equal(get_rse_attribute(key=rse, rse_id=get_rse_id(rse)), [True])
-
         rse = rse_name_generator()
         renamed_rse = 'renamed_rse%s' % rse
         ret = self.client.add_rse(rse)
